%%                          Wiener Filter Predictions
% Alistair Boettiger                                   Date Begun: 03/14/08
% Getz Lab                                       Functional Since: 04/23/08
% Functionally Complete                             Last Modified: 08/01/09
%
% 
%  
%
%
%
% Training Inputs
%   - Movement COM for all elepants (lat, long)                 2*K vars
%   - Dispersion character  "     "        " (bounding ellipse) 2*K vars
%   - season / average NDVI                                     1 var
%   - COM distance to water, road, park                         3*K vars
%   - NDVI torus-sector profile for given inner & outer radius  9*K vars     
%   - Absolute time                                             1 var
%   
%
% Prediction Outputs
%   * Center of mass of future movement sequence (lat, long)    2*K vars
%   * Dispersion character "     "        " (bounding ellipse)  2*K vars
%   - NDVI sector score of chosen region                        1*K vars
%
% This Program calls custom functions:
%           mapo1t1.m, bnd_ellipse_multi.m, NDVI_torus.m
% This Script requires data files
%          E_data.mat, NDVIs.mat, ave_ndvi.mat
%          optional: e_stats.mat.  Overwrites: temp.mat
% The data imported to this program is preprocessed using:
%           import_NDVI.m, sample_n_pair_data.m
% The data organized by this script is analyzed by 
%          Wiener_filter_generalize_v2.m




%Notes:
%  Modified to include Anastesia 07/24/09
% Modified to include absolute time 07/27/09
% Modified to save  score of NDVI sector 08/01/09
%
%
% Prep Static and NDVI data for Wiener filter analysis
% Wiener filter data organization and normalization
% loop through all elphants do all sims
% Modified from density_multi_loop.m
% Fatal flaw on NDVI fixed 4/21 3:30



clear all; close all;

hrs = 24; % number of hours used for position density calc.
hr = 1;  % reduce sampling frequency
his = hrs/hr; 

% ~~~~~~~~~~~~~Import Elephant Tracking data~~~~~~~~~~~~~~ %
load E_data % data on 13 elephants from 01/13/00 to 06/25/07, sampled hourly  
load NDVIs % NDVI data with hourly index reference and lat and long references


K=13; % number of pachederms 
NDVI_mov = cell(1,13);   % store score of sectors to which elephants move

% seperate out lat and long data for the first K elephants
Elats = data(:,1:2:2*K); Elongs = data(:,2:2:2*K);
NDVI = 2*double(NDVI)/255-1; % map NDVI to [0,1]


% Compute COMs and bounding ellipses
%  [xcm, ycm, ra, rb, o] = bnd_ellipse_multi(Elongs,Elats,K,his);
 % %save e_stats4 xcm ycm ra rb o % save results from COM tracking
 load e_stats4 xcm ycm ra rb o  % import results from COM tracking
T2 = length(xcm) - 2*his; % length of inputs and outputs 

time = 1:length(xcm);  
timen = mapo1t1(time); 

% find time at which each elephant starts recording and stops recording
[jnk t1] = max(xcm>0); % sets all nonzero elements to 1, max returns first of equivelant maxes 
[jnk tend] = max(flipud(xcm)>0);
tend = length(xcm)-tend+1;


% figure(1); clf; plot(xcm);
% hold on; plot(t1,ones(1,12),'r.');
% hold on; plot(tend,ones(1,12),'c.');
% legend('Goya','Maya','Stratus','Ngalatoni','Neptune','Aztec','Maua',...
% 'Amina','Ndorobo','Monsoon','Jerusalem','Rosemary');


    % convert inputs with missing data to NaN flags
    %     (this avoids upsetting min max rescaling, difference calcs, etc)
    Xcm = xcm;  Xcm(xcm==0)=NaN;
    Ycm = ycm;  Ycm(ycm==0)=NaN;
    Ra = ra;  Ra(ra==0)=NaN;
    Rb = ra;  Rb(rb==0)=NaN;
    O = o;  O(o==0)=NaN;

 
for E = 1:K     % loop through elephant on which to center analysis
     Ti = t1(E); % restrict time to interval for choice elephant
     Te = tend(E); %
   
       


%------------------------- Data Preprocessing ----------------------------%
    
    
   % Normalize the choice elephant's stats too using same norm for all Es
   
       lx = min(min(Xcm));
       ux = max(max((Xcm-lx)));
       xcn =  2*((Xcm(Ti:Te,E)-lx)/ux-.5); 
       
       ly = min(min(Ycm));
       uy = max(max((Ycm-ly)));
       ycn =  2*((Ycm(Ti:Te,E)-ly)/uy-.5); 
       
       lr = min(min(Ra));
       ur = max(max((Ra-lr)));
       ran =  2*((Ra(Ti:Te,E)-lr)/ur-.5); 
       
       lo = min(min(O));
       uo = max(max((O-lo)));
       on =  2*((O(Ti:Te,E)-lo)/uo-.5);    



 % % Normalize to just this Elephants
%     xcn = mapo1t1(Xcm(Ti:Te,E));
%     ycn = mapo1t1(Ycm(Ti:Te,E));
%     ran = mapo1t1(Ra(Ti:Te,E));
%     rbn = mapo1t1(Rb(Ti:Te,E));
%     on = mapo1t1(O(Ti:Te,E));
    % map NaNs to 0  zero always carrys no weight so these should be safe.  The nonlinear filters can make almost zero look like something very different 
    xcn(isnan(xcn)) = 0; 
    ycn(isnan(ycn)) = 0; 
    ran(isnan(ran)) = 0; 
%    rbn(isnan(rbn)) = 0; 
    on(isnan(on)) = 0; 
    %------------------------------------------------------------------%
 
         % ~~~~~~~~~~~~~~~~Import static data~~~~~~~~~~~~~~~~~~~~~
    load dist_quant; % static envi. measures
    % locate elephant motions on static-data raster maps
    yis = find(Slat> min(Ycm(Ti:Te,E)) & Slat< max(Ycm(Ti:Te,E)) );
    xis = find(Slong> min(Xcm(Ti:Te,E)) & Slong< max(Xcm(Ti:Te,E)) );
    grid_long = Slong(xis); % even grid points in degrees
    grid_lat = Slat(yis); 
    D2W = DtoWater2(yis(1):yis(end),xis(1):xis(end));
    D2P = DtoPark2(yis(1):yis(end),xis(1):xis(end));
    D2R = DtoRoad2(yis(1):yis(end),xis(1):xis(end));
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
     
    
%~~~~~~~~~~~~~~~~~~~~~Optional: Include NDVI for chosen E~~~~~~~~~~~~~~~~~%
    
  % initialize appropriate variables. 
    ri = 10; % 1
    ro = 20; % 6km optimal NDVI perception chosen from NDVI studies
    segs = 9; % number of segmeents in which to divide NDVI values
    T = Te-Ti; 
    scor = zeros(T,1); sec_var=zeros(T,1); mean_ndvi = zeros(T,segs) ;
 d2r = zeros(T,1); d2w=d2r; d2p=d2r;    
 
for j=1:T+1;
    t=Ti+j-1;
  % compute NDVI mean sector values,
     [jnk xc] = min(abs(long' - Xcm(t,E) )); % find closest grid point
     [jnk yc] = min(abs(lat' - Ycm(t,E) )); % j is current time
     [jnk xf] = min(abs(long' - Xcm(t+his) )); % j+his first nonoverlapping future time
     [jnk yf] = min(abs(lat' - Ycm(t+his) )); 
     i = Nind(t+his-1);     
[mean_ndvi(j,:), scor(j), sec_var(j)] = NDVI_torus(NDVI(:,:,i),segs,xc,yc,ri,ro,xf,yf);
%   % Static data analysis 
     [xer xposi] = min( abs( grid_long - Xcm(t,E) ) ); % find closest grid point to each K elephant points 
     [yer yposi] = min( abs( grid_lat - Ycm(t,E)  ) );
    d2r(j) = D2R(yposi,xposi);  % read the value of the Envi var at the nearest grid point to the elephant position 
    d2w(j) = D2W(yposi,xposi); % note lat and long need to be paired in time, so we only want the diag entries
    d2p(j) = D2P(yposi,xposi);
%   j/T % progress ticker

end    
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%
    
% time stuff
 load ave_ndvi;  season = ave_ndvi(Ti:Te); 
 

% in = [xcn ycn ran on season d2r d2w d2p mean_ndvi timen(Ti:Te)'];
in = [xcn ycn ran on season d2r d2w d2p mean_ndvi];

NDVI_mov{E} = [scor,sec_var]; 


% % Flag start and end points of gaps
%     flag = xcn(:,1)==0;  flag = flag(2:end)-flag(1:end-1);
%     sis = find(flag==1); % start points of gaps
%     ses = find(flag==-1); % end points of gaps
%     ses = [0; ses]; % we want to pull out the data not the gaps
%     sis = [sis; length(in(:,1))]; % 0 to 1st gap start is the first data segment 
%      % plot data and mark gaps
%         figure(1); clf; plot(xcn);
%         hold on; plot(sis, zeros(1,length(sis)),'c.'); 
%         plot(ses,zeros(1,length(sis)),'r.');
%  
%         N=length(xcn);
%     segs = length(sis);
%     B = 1000; % batch size;
%     
%     kk=0;
%     for j=1:segs
%         temp = in(1+ses(j):sis(j),:);  % split out across gaps
%         JJ = ceil(length(temp)/B);
%         for jj=1:JJ % separate gap sections into regions at most B long
%             kk=kk+1;
%             sys_dat{kk}=  temp(1+(jj-1)*B:min(jj*B,length(temp)),:);
%         end
%     end
%         E
%     
     %  save(['E_all4_', num2str(E), '_wiener'],'sys_dat'); clear sys_dat;
end % end loop through elephants
 

save ndvi_freq NDVI_mov


names = {'Goya','Maya','Stratus','Ngalatoni','Neptune','Aztec','Maua',...
 'Amina','Ndorobo','Monsoon','Jerusalem','Rosemary','Anastesia'};
rchosen = [];
figure(1);  set(gcf,'color','w'); clf;
for E=1:13
    rchosen = [rchosen; NDVI_mov{1,E}(:,1)];
    figure(1);  subplot(4,4,E); hist(NDVI_mov{1,E}(:,1),9);
    title(names{E});
    xlabel('stay, highest,        lowest');
end
    figure(2); clf; hist(rchosen,9);
 title('NDVI sector chosen');
 set(gcf,'color','w');
   