%%                     Binary Metric Predictions
% Alistair Boettiger                                   Date Begun: 01/28/08
% Getz Lab                                       Functional Since: 02/15/08
% Functionally Complete                             Last Modified: 02/21/08
%
%
%   binary_targets_ndvi.m
%
%Notes:
%    Binary metric classifications:
%       *  does the elephant stay within d km of it's original position
%            over the  next hr hours?
%       *  does the animal move towards or away from park/road/water
%       *  does the animal move towards higher or lower ndvi values
%       *  does the animal end up at higher or lower ndvi values
%       -  do the elephants get closer or further apart
%   Other classifications
%       *  what ndvi-ranked octant does the elephant move into 1-8
%       *  mean displacement of next stp moves
%       *  std displacement of next stp moves
%       *  mean and std angles
%
% This Program calls custom functions:
%           map0t1.m, splitsets.m, NDVI_sector.m
% The data imported to this program is preprocessed using:
%           import_NDVI.m, sample_n_pair_data.m



clear all; 

% Fundamental Control Parameters 
hr = 8; % sampling interval
his = 10; % previous steps used for training
stp = 6; % future step at which predictions are made
S = 10; % number of NDVI grid points to integrate over
K = 6; % number of steps to provide envi data at  

% ~~~~~~~~~~~~~Import Elephant Tracking data~~~~~~~~~~~~~~ %
    load J_dat; ELat = J{1}; ELong = J{2}; Date = J{3};   

    ex = ELong(1:hr:end);  
    ey = ELat(1:hr:end);
    % ex2 = C{1,2}(1:hr:end); ey2=C{1,1}(1:hr:end);
    % find domain corners  (good for lots of stuff);
    latL = min(ey); latU = max(ey); 
    longL = min(ex); longU = max(ex);

     % map inputs to -1 to 1
     [exn exc] = map0t1(ex); 
     [eyn eyc] = map0t1(ey);
    
%  %  train on displacements not positions
%     [exn exc] = map0t1(diff(ex));
%     [eyn,eyc] = map0t1(diff(ey));
%     
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ %

% ~~~~~~~~~~~~~~~~Import static data~~~~~~~~~~~~~~~~~~~~~
    load dist_quant; % static envi. measures
    % locate elephant motions on static-data raster maps
    yis = find(Slat>latL & Slat< latU);
    xis = find(Slong> longL & Slong< longU);
    grid_long = Slong(xis); % even grid points in degrees
    grid_lat = Slat(yis); 
    D2W = DtoWater2(yis(1):yis(end),xis(1):xis(end));
    D2P = DtoPark2(yis(1):yis(end),xis(1):xis(end));
    D2R = DtoRoad2(yis(1):yis(end),xis(1):xis(end));
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~%



% Binary classification parameters
    d = .04; % min distance (in degrees) for 'leaving area'
    bw = d/20; % bias for movement from water
    br = d/8; % bias for movement from road
    bp = d/8;% bias for movement from park
    be = d/4; % bias for relative elephant motion
    an = 4; % km squares to average up and down for current ndvi
    gn=5;  % km squares to average to estimate global ndvi
    bn = .005; % bias to qualify as different ndvi  
    
 
% data structures for neural net training
    T = length(ex) - his -stp;
% input descriptions 
    d2r = zeros(T,K); d2w = zeros(T,K); d2p = zeros(T,K); 
    sind = ndvi_ind(1:hr:end); % reduce NDVI hourly samples to chosen interval
    nave = zeros(T,8);
    All_ndvi = zeros(T,(2*S+1)^2);
    NDVI = 2*double(NDVI)/255-1; % map NDVI to [-1,1]
% global structures
    zT = zeros(1,T); % shorthand
    num_outs = 1+5; % number of output classifications to hit.
    num_ins = 2*his+1 + 3*K;  % position steps + time + stat vars ...
    in = zeros(T,num_ins);
    out = zeros(T,num_outs);
% output binary behavior descriptions
    stays=zT; towater=zT; fromwater=zT; topark=zT; frompark=zT; toroad=zT; fromroad=zT; 
    ndvi_up=zT; ndvi_down=zT; ndvi_toup=zT; ndvi_todown=zT; ndvi_gup=zT; ndvi_gdown=zT; 
    % att=zT; rpl=zT;
% output continuous valued behavioral descriptions 
    ndvi_quad=zT; mdis=zT; sdis=zT; % mang=zT; sang=zT;


% assemble data structures
for j=1:T;
 %----------------Analysis for Input Data--------------------------%     
 % convert elephant positions to NDVI gridpoints
 % current position is j+hist-1.  1st given position is j. 
     [xer xpos] = min(abs(long' - ex(j+his-1))); % find closest grid point to each K elephant points 
     [yer ypos] = min(abs(lat' - ey(j+his-1))); % lat and long are vecs of NDVI grid points 
 % average NDVI of appropriate date in given direction around elephant
    i = sind(j+his-1); 
    nave(j,:) = NDVI_sector(NDVI(:,:,i),xpos,ypos,S);
    %      all_ndvi = NDVI(ypos-S:ypos+S,xpos-S:xpos+S,i);
    %      All_ndvi(j,:) = reshape(all_ndvi,1,(2*S+1)^2);
%  when was the elephant last in a high NDVI region?

%  when was the elephant last in a low NDVI region? 

% when was the elephant last near water?

time  = (j+his-1)/T;
in1(j,:) = [eyn(j:j+his-1)', exn(j:j+his-1)',time] ; % input positions based on history     

    %^^^^^^train on Static Envi Stats of most recent k points^^^^^%
   if K~=0
    Grid_long = []; Grid_lat = [];
    for k=1:K
        Grid_long = [Grid_long; grid_long]; % create rpt array of grid points to do 1 step substraction
        Grid_lat = [Grid_lat; grid_lat]; 
    end
    Expos = meshgrid(ex(his+j-K:his-1+j), grid_long); % create rpt array of elephant positions to do 1 step substraction 
    Eypos = meshgrid(ey(his+j-K:his-1+j), grid_lat);
     [xer xposi] = min(abs(Grid_long' - Expos)); % find closest grid point to each K elephant points 
     [yer yposi] = min(abs(Grid_lat' - Eypos));
     
    d2r(j,:) = diag(D2R(yposi,xposi));  % read the value of the Envi var at the nearest grid point to the elephant position 
    d2w(j,:) = diag(D2W(yposi,xposi)); % note lat and long need to be paired in time, so we only want the diag entries
    d2p(j,:) = diag(D2P(yposi,xposi));
    in2(j,:) = [in1(j,:),d2r(j,:),d2w(j,:),d2p(j,:)]; % concatinate into a big input vector 
   else
    in2(j,:) = in1(j,:);
   end 
    %^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^%   
  %-----------------------------------------------------------------------%  
  
  %---------------------Analysis for Output Data--------------------------%
    % does the elephant stay within d km of original position?
    stays(j) = min(sqrt( (ex(j+his:j+his+stp-1)-ex(j)).^2 + (ey(j+his:j+his+stp-1)-ey(j)).^2 ) < d);
    % locate elephant on static map coordinate
       [xer xpos0] = min( abs(ex(j+his)-grid_long) );
       [yer ypos0] = min( abs(ey(j+his)-grid_long) );
       [xer xposS] = min( abs(ex(j+his+stp-1)-grid_long) );
       [yer yposS] = min( abs(ey(j+his+stp-1)-grid_long) );
    % towards or away from object?    
       towater(j) = D2W(yposS,xposS) + bw < D2W(ypos0,xpos0);
       fromwater(j) = D2W(yposS,xposS) > bw + D2W(ypos0,xpos0);
       topark(j) = D2P(yposS,xposS) + bp < D2P(ypos0,xpos0);
       frompark(j) = D2P(yposS,xposS) > bp + D2P(ypos0,xpos0);
       toroad(j) = D2R(yposS,xposS) + br < D2R(ypos0,xpos0);
       fromroad(j) = D2R(yposS,xposS) > br + D2R(ypos0,xpos0);
%     % towards or away from other elephant
%         d0 =( ex(j+his) - ex2(j+his)).^2 + (ey(j+his)-ey2(j+his)).^2;
%         dS =( ex(j+his+stp-1) - ex2(j+his+stp-1)).^2 + (ey(j+his+stp-1)-ey2(j+his+stp-1)).^2;
%         att(j) = dS + be < d0;
%         rpl(j) = dS > be + d0; 
     % towards higher or lower average ndvi?
        i2 = sind(j+his+stp-1); % ndvi time index at projected time
            [xer xpos2] = min(abs(long' - ex(j+his+stp-1))); % find closest grid point to each K elephant points 
            [yer ypos2] = min(abs(lat' - ey(j+his+stp-1))); 
        ndvi_up(j) = mean(mean(NDVI(ypos-an:ypos+an,xpos-an:xpos+an,i))) + bn < mean(mean(NDVI(ypos2-an:ypos2+an,xpos2-an:xpos2+an,i2))); % compare local ndvi scores now and later (region and time)
        ndvi_down(j) = mean(mean(NDVI(ypos-an:ypos+an,xpos-an:xpos+an,i))) > bn + mean(mean(NDVI(ypos2-an:ypos2+an,xpos2-an:xpos2+an,i2)));        
        ndvi_toup(j) = mean(mean(NDVI(ypos-an:ypos+an,xpos-an:xpos+an,i))) + bn < mean(mean(NDVI(ypos2-an:ypos2+an,xpos2-an:xpos2+an,i))); % compare local ndvi scores now between regions
        ndvi_todown(j) = mean(mean(NDVI(ypos-an:ypos+an,xpos-an:xpos+an,i))) > bn + mean(mean(NDVI(ypos2-an:ypos2+an,xpos2-an:xpos2+an,i)));  
        ndvi_gup(j) = mean(mean(NDVI(ypos-gn:ypos+gn,xpos-gn:xpos+gn,i))) + bn < mean(mean(NDVI(ypos-gn:ypos+gn,xpos-gn:xpos+gn,i2)));   % sig. global increase in NDVI
        ndvi_gdown(j) = mean(mean(NDVI(ypos-gn:ypos+gn,xpos-gn:xpos+gn,i))) > bn + mean(mean(NDVI(ypos-gn:ypos+gn,xpos-gn:xpos+gn,i2)));  % sig. global decrease in NDVI
  %% which vegitation sector did it move into?  
        %%  based on input vegitation sectors, nave  
 rate = sortrows([nave(j,:)',[1:8]']);  % score the sectors, keep track of original numbering 
tmp = .0089*S; theta = (pi/4:pi/4:2*pi)-pi/8;
test_x = ex(j+his) + tmp*cos(theta);
test_y = ey(j+his) + tmp*sin(theta);    
  [er iq] = min( (ex(j+his+stp) - test_x).^2 + (ey(j+his+stp) - test_y).^2 ) ;
 ndvi_quad(j) = find(rate(:,2)==iq); % score of 1 to 8 lowest to highest of the sector elephant moved into 
  %  figure(3); clf; plot(test_x,test_y,'k.'); hold on; plot(ex(j+his+stp),ey(j+his+stp),'r.');  
 ndvi_quad_norm = 2*(ndvi_quad(j)-1)/7-1; 

 % statistics of next stp moves
 mdis(j) = log(mean(diff(ex(j+his:j+his+stp)).^2+diff(ey(j+his:j+his+stp)).^2));
 sdis(j) = log(std(diff(ex(j+his:j+his+stp)).^2+diff(ey(j+his:j+his+stp)).^2));
% mang(j) = log(mean( atan(diff(ex(j+his:j+his+stp))./diff(ey(j+his:j+his+stp))) ));
% sang(j) = log(std( atan(diff(ex(j+his:j+his+stp))./diff(ey(j+his:j+his+stp))) ));
 
     %out(j,:) = [stays(j) towater(j) fromwater(j) topark(j) frompark(j) toroad(j) fromroad(j)];  
      out(j,:) = [stays(j) towater(j) fromwater(j)  ndvi_up(j) ndvi_down(j) ndvi_quad_norm];  
end

[mdisn mdisc] = map0t1(mdis);
[sdisn stdic] = map0t1(sdis);
out = [out, mdisn', sdisn'];  num_outs = num_outs+2;

% concatinate different input options
    in = [in2, nave]; num_ins = num_ins +8;
  %   in = [in, All_ndvi]; num_ins = num_ins + (2*S+1)^2; 
   
  
snave = find(std(nave')>.1);
snave = find(mean(nave') < -.2 & std(nave')>.1);
figure(3); clf; hist(ndvi_quad(snave),8);
size(snave);
figure(4); clf; hist(ndvi_quad,8);
xlabel('mean region NDVI, sorted minimum to maximum');
ylabel('frequency region was chosen');
%   
% 
% % split data and train neural network
% [train_set,v_set,ts_set] = splitsets(in, out);
% net = newff([-1*ones(num_ins,1),ones(num_ins,1)],[20,num_outs]);
% [net,tr] = train(net, train_set.P, train_set.T,[],[],v_set,ts_set);
% 
% 
% ts_p = sim(net, ts_set.P);
% v_p = sim(net, v_set.P);
% 
% v_error =  sum(abs( round(v_p) - v_set.T),2 )/length(v_set.T)
% v_ern = v_error./ ( sum(out)/length(out))';
% cont_err = mean(abs((v_p(6:end,:) - v_set.T(6:end,:))'));
% v_ern(6:end) = cont_err 
% 
% guess_var = sum(round(v_p(1:5,:)'))'/length(v_p(1:5,:)')'

 
